pub enum IntoValueError<H> {
NotUnique(H),
Extract {
value: H,
error: GroupError,
},
}Expand description
Error returned when a value cannot be consumed without changing its owner.
Examples
use tenferro_ad::{EagerRuntime, EagerTensor, IntoValueError, Tensor};
use tenferro_cpu::CpuBackend;
let ctx = EagerRuntime::with_cpu_backend(CpuBackend::new())?;
let value = EagerTensor::from_tensor_in(
Tensor::from_vec_col_major(vec![1], vec![1.0_f64])?,
ctx,
)?;
let _shared = value.clone();
assert!(matches!(
value.into_value(),
Err(IntoValueError::NotUnique(_))
));Variants
NotUnique(H)
Another eager handle, tape record, or checkpoint retains the value.
Extract
Group extraction failed after the handle was uniquely acquired.