apply_standard_op

Function apply_standard_op 

pub fn apply_standard_op(
    op: StdTensorOp,
    inputs: &[&EagerTensor],
) -> Result<EagerTensor>
Apply one standard tensor op eagerly and record it for AD when needed.

Extension crates use this when an extension-level eager operation expands into ordinary StdTensorOp nodes instead of a custom extension primitive.

Errors

Returns tenferro_runtime::Error::TensorRuntime containing tenferro_tensor::ValidationError::InvalidArgument if an extension op is passed to this standard-op entry point. Returns tenferro_runtime::Error::ContextMismatch for tensors from different eager contexts and propagates typed tensor/backend/runtime-state failures from the selected eager context.