Struct ExtensionCacheLimits
pub struct ExtensionCacheLimits { /* private fields */ }Expand description
Bounded retention limits for extension runtime caches.
Implementations
impl ExtensionCacheLimits
impl ExtensionCacheLimits
pub const fn new(max_entries: NonZero<usize>) -> ExtensionCacheLimits
pub const fn new(max_entries: NonZero<usize>) -> ExtensionCacheLimits
Build limits from a maximum entry count and the default byte bound.
Examples
use std::num::NonZeroUsize;
use tenferro_runtime::ExtensionCacheLimits;
let limits = ExtensionCacheLimits::new(NonZeroUsize::new(4).unwrap());
assert_eq!(limits.max_entries().get(), 4);
assert!(limits.max_retained_bytes().is_some());pub const fn max_entries(self) -> NonZero<usize>
pub const fn max_entries(self) -> NonZero<usize>
Maximum entries retained by the store.
pub const fn max_retained_bytes(self) -> Option<NonZero<usize>>
pub const fn max_retained_bytes(self) -> Option<NonZero<usize>>
Maximum logical retained bytes, when configured.
pub const fn with_max_retained_bytes(
self,
max_retained_bytes: NonZero<usize>,
) -> ExtensionCacheLimits
pub const fn with_max_retained_bytes( self, max_retained_bytes: NonZero<usize>, ) -> ExtensionCacheLimits
Return limits with a new logical retained-byte bound.
Examples
use std::num::NonZeroUsize;
use tenferro_runtime::ExtensionCacheLimits;
let limits = ExtensionCacheLimits::default()
.with_max_retained_bytes(NonZeroUsize::new(1024).unwrap());
assert_eq!(limits.max_retained_bytes().unwrap().get(), 1024);Trait Implementations
impl Clone for ExtensionCacheLimits
impl Clone for ExtensionCacheLimits
fn clone(&self) -> ExtensionCacheLimits
fn clone(&self) -> ExtensionCacheLimits
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreimpl Debug for ExtensionCacheLimits
impl Debug for ExtensionCacheLimits
impl Default for ExtensionCacheLimits
impl Default for ExtensionCacheLimits
fn default() -> ExtensionCacheLimits
fn default() -> ExtensionCacheLimits
Returns the “default value” for a type. Read more
impl PartialEq for ExtensionCacheLimits
impl PartialEq for ExtensionCacheLimits
fn eq(&self, other: &ExtensionCacheLimits) -> bool
fn eq(&self, other: &ExtensionCacheLimits) -> bool
Tests for
self and other values to be equal, and is used by ==.impl Copy for ExtensionCacheLimits
impl Eq for ExtensionCacheLimits
impl StructuralPartialEq for ExtensionCacheLimits