pub enum SemanticExtensionRegistryError {
DuplicateRule {
family_id: &'static str,
role: SemanticAdRuleRole,
},
MalformedFamilyId {
family_id: &'static str,
},
}Expand description
Registration failures for semantic extension AD rules.
Variants
DuplicateRule
A rule with the same family and role is already present.
Fields

role: SemanticAdRuleRoleDuplicate semantic AD role.
MalformedFamilyId
A rule family is not a namespaced, versioned identifier.