pub struct AdTransformCacheLimits { /* private fields */ }Expand description
Retention limits for AD transform graph caches.
The retained-byte limit is a logical payload estimate, not process RSS.
Examples
use std::num::NonZeroUsize;
use tenferro_ad::AdTransformCacheLimits;
let limits = AdTransformCacheLimits::new(NonZeroUsize::new(4).unwrap());
assert_eq!(limits.max_entries().get(), 4);
assert!(limits.max_retained_bytes().is_some());Implementations
Sourceimpl AdTransformCacheLimits
impl AdTransformCacheLimits
Sourcepub fn new(max_entries: NonZeroUsize) -> Self
pub fn new(max_entries: NonZeroUsize) -> Self
Create AD transform cache limits with the default retained-byte bound.
Examples
use std::num::NonZeroUsize;
use tenferro_ad::AdTransformCacheLimits;
let limits = AdTransformCacheLimits::new(NonZeroUsize::new(2).unwrap());
assert_eq!(limits.max_entries().get(), 2);Sourcepub fn max_entries(self) -> NonZeroUsize
pub fn max_entries(self) -> NonZeroUsize
Return the maximum number of retained AD transform entries.
Examples
use tenferro_ad::AdTransformCacheLimits;
assert!(AdTransformCacheLimits::default().max_entries().get() > 0);Sourcepub fn max_retained_bytes(self) -> Option<NonZeroUsize>
pub fn max_retained_bytes(self) -> Option<NonZeroUsize>
Return the logical retained-byte bound, when one is configured.
Examples
use tenferro_ad::AdTransformCacheLimits;
assert!(AdTransformCacheLimits::default().max_retained_bytes().is_some());Sourcepub fn with_max_retained_bytes(self, max_retained_bytes: NonZeroUsize) -> Self
pub fn with_max_retained_bytes(self, max_retained_bytes: NonZeroUsize) -> Self
Return limits with a new logical retained-byte bound.
Examples
use std::num::NonZeroUsize;
use tenferro_ad::AdTransformCacheLimits;
let limits = AdTransformCacheLimits::default()
.with_max_retained_bytes(NonZeroUsize::new(1024).unwrap());
assert_eq!(limits.max_retained_bytes().unwrap().get(), 1024);Trait Implementations
Sourceimpl Clone for AdTransformCacheLimits
impl Clone for AdTransformCacheLimits
Sourcefn clone(&self) -> AdTransformCacheLimits
fn clone(&self) -> AdTransformCacheLimits
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for AdTransformCacheLimits
impl Debug for AdTransformCacheLimits
Sourceimpl Default for AdTransformCacheLimits
impl Default for AdTransformCacheLimits
Sourceimpl PartialEq for AdTransformCacheLimits
impl PartialEq for AdTransformCacheLimits
Sourcefn eq(&self, other: &AdTransformCacheLimits) -> bool
fn eq(&self, other: &AdTransformCacheLimits) -> bool
Tests for
self and other values to be equal, and is used by ==.impl Copy for AdTransformCacheLimits
impl Eq for AdTransformCacheLimits
impl StructuralPartialEq for AdTransformCacheLimits