pub struct EagerNoGradGuard { /* private fields */ }Expand description
Scope guard that temporarily disables eager operation recording.
Values computed while this guard is alive are concrete eager tensors, but they do not participate in reverse-mode gradient tracking.
Examples
use tenferro_ad::{EagerRuntime, EagerTensor, Tensor};
use tenferro_cpu::CpuBackend;
let ctx = EagerRuntime::with_cpu_backend(CpuBackend::new());
let x = EagerTensor::requires_grad_in(
Tensor::from_vec_col_major(vec![1], vec![2.0_f64]).unwrap(),
ctx.clone(),
)?;
let y = {
let _guard = ctx.no_grad();
x.mul(&x)?
};
assert!(!y.tracks_grad());Trait Implementations
Sourceimpl Debug for EagerNoGradGuard
impl Debug for EagerNoGradGuard
Sourceimpl Drop for EagerNoGradGuard
impl Drop for EagerNoGradGuard